/*	freeems.h

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

/* Header file multiple inclusion protection courtesy eclipse Header Template	*/
/* and http://gcc.gnu.org/onlinedocs/gcc-3.1.1/cpp/ C pre processor manual		*/
#ifndef FILE_FREEEMS_H_SEEN
#define FILE_FREEEMS_H_SEEN

#include "9s12xdp512.h"

/* Common macro definitions across the project */
/* Boolean */
#define TRUE 1
#define FALSE 0

/* 8 bit values */
#define ZEROS 0x00
#define ONES 0xFF

/* 16 bit values */
#define ZEROS16 0x0000 /* for consistency only */
#define ONES16 0xFFFF

/* Individual bits */
#define BIT0	0x0001 /* 1st bit	= 1		*/
#define BIT1	0x0002 /* 2nd bit	= 2		*/
#define BIT2	0x0004 /* 3rd bit	= 4		*/
#define BIT3	0x0008 /* 4th bit	= 8		*/
#define BIT4	0x0010 /* 5th bit	= 16	*/
#define BIT5	0x0020 /* 6th bit	= 32	*/
#define BIT6	0x0040 /* 7th bit	= 64	*/
#define BIT7	0x0080 /* 8th bit	= 128	*/

#define BIT8	0x0100 /* 9th bit	= 256	*/
#define BIT9	0x0200 /* 10th bit	= 512	*/
#define BIT10	0x0400 /* 11th bit	= 1024	*/
#define BIT11	0x0800 /* 12th bit	= 2048	*/
#define BIT12	0x1000 /* 13th bit	= 4096	*/
#define BIT13	0x2000 /* 14th bit	= 8192	*/
#define BIT14	0x4000 /* 15th bit	= 16384	*/
#define BIT15	0x8000 /* 16th bit	= 32768	*/

/* Definitions for functions used across the project */
/* http://www.seattlerobotics.org/encoder/200109/encoder.htm section Assembly */
#define INLINE_ASSEMBLY(code) __asm__ __volatile__ (code)

/* GLOBAL Variables */

/* Setting variables */
unsigned char ports;								/* How many groups of injectors we are firing			*/
unsigned char coils;								/* How many coils we are firing							*/

unsigned char combustionEventsPerEngineCycle;		/* How many power producing events per engine cycle		*/
unsigned char revolutionsPerEngineCycle;			/* Rotary = 1, 2 Stroke = 1, 4 Stroke = 2				*/

unsigned short capacityOfAirPerCombustionEvent;		/* How much air it swallows per power producing event	*/
unsigned short perPrimaryInjectorChannelFlowRate;	/* How much fuel flows per minute per channel			*/
unsigned short perSecondaryInjectorChannelFlowRate;	/* How much fuel flows per minute per channel			*/
unsigned char stagedInjection;						/* Whether to use the staged outputs 0 = false			*/


/* State variables */
unsigned char rpmValid;		/* Whether we are sure rpm is what the variable says (used to inject fuel without ignition below the threshold rpm) : 0 = false */
unsigned char crankSync; 	/* Wasted spark/Semi sequential : 0 = false */
unsigned char camSync;		/* COP/Full sequential : 0 = false */
unsigned char phase;		/* Which half of the engine cycle we are in : 0 = false */


/* Engine runtime properties */
unsigned short rpm;			/* Revolutions Per Minute			: 0		- 65,536		(1 RPM)					*/
unsigned short clt;			/* Coolant Temperature				: 0.0	-    511.5		(0.5 Celsius (/2 -100))		*/
unsigned short map;			/* Manifold Absolute Pressure		: 0.0	-    511.5		(0.5 kPa (/2))			*/
unsigned short iat;			/* Inlet Air Temperature			: 0.0	-    102.3		(0.1 Celsius (/10,-50))	*/
unsigned short tps;			/* Throttle Position Sensor			: 0.0	-    102.3		(0.1 % (/10))			*/
unsigned short ego;			/* Exhaust Gas Oxygen				: 0.000	-      2.046	(0.002 lambda (/500))	*/


/* Output variables */
unsigned char outputbank;	/* Which bank of output variables to use for logging and/or calculations etc */
unsigned short rpmOut1;		/*  */
unsigned short cltOut1;
unsigned short mapOut1;
unsigned short iatOut1;
unsigned short tpsOut1;
unsigned short egoOut1;




#else
	/* let us know if we are being untidy with headers */
	#warning "Header file FREEEMS_H seen before, sort it out!"
/* end of the wrapper ifdef from the very top */
#endif
